import Vue from 'vue'
import messageModel from './Message.vue'
import Utils from '../../utils/common'

const MessageConstructor = Vue.extend(messageModel)
const defaultOptions = {
  message: '',
  showClose: false,
  more: false
}
const instances = []
let seed = 1

const Message = (message = '', options = {}) => {
  return new Promise((resolve, reject) => {
    if (['info', 'success', 'warning', 'error'].indexOf(options.type) === -1) {
      options.type = 'info'
    }
    if (['top', 'middle', 'bottom', 'right'].indexOf(options.position) === -1) {
      options.position = 'middle'
    }
    options.handleClose = () => {
      Message.close(instance.id, resolve)
    }
    const instance = new MessageConstructor({
      el: document.createElement('div'),
      data: Object.assign({}, defaultOptions, options)
    })
    instance.id = `message__${seed++}`
    document.body.appendChild(instance.$el)
    if (options.more) {
      const { openText, closeText, content, height = `200px` } = options.more
      const h = instance.$createElement
      const el = instance.$el
      const showClose = instance.showClose ? ' zp-msg-more-text-with-close' : ''
      const scrollableContent = h('zp-scroller', {
        style: {
          height
        },
        attrs: {
          class: 'zp-msg-scroller'
        }
      }, [content])
      instance.$slots.default = [scrollableContent]
      el.classList.add('zp-msg-with-more')
      el.querySelector('.zp-msg-container div').insertAdjacentHTML('beforeend', `<span class="zp-msg-more-text${showClose}">${openText}</span>`)
      el.querySelector('.zp-msg-more-text').addEventListener('click', evt => {
        instance.showMore = !instance.showMore
        setTimeout(() => {
          el.querySelector('.zp-msg-more-text').textContent = instance.showMore ? closeText : openText
        }, 0)
      })
    }
    instance.message = options.length ? Utils.substring(message, options.length) : message
    instance.visible = true
    instances.push(instance)
  })
}

Message.close = (id, resolve) => {
  const len = instances.length
  for (let i = 0; i < len; i++) {
    if (id === instances[i].id) {
      instances.splice(i, 1)
      try {
        resolve(instances[i])
      } catch (e) {
        return false
      }
      break
    }
  }
}

Message.closeAll = () => {
  for (let i = instances.length - 1; i >= 0; i--) {
    instances[i].closed = true
  }
}

export default Message
