export default {
  isObject (obj) { // 是否为对象
    return (!!obj) && (obj.constructor === Object)
  },
  isEmptyObject (obj) { // 是否为空对象
    return this.isObject(obj) && (Object.keys(obj).length === 0)
  },
  isNumber (n) { // 是否为数字
    return !isNaN(parseFloat(n)) && isFinite(n)
  },
  isDate (d) { // 是否为日期
    // if (d === null || d === undefined) {
    //   return false
    // }
    // if (isNaN(new Date(d).getTime())) { 
    //   return false
    // }
    // return true
    return d instanceof Date && !isNaN(d)
  },
  isDef (val) { // 是否已定义
    return val !== undefined && val !== null
  },
  isBoolean (val) { // 是否为布尔值
    return !!(typeof val === 'boolean')
  },
  isFunction (val) { // 是否为函数
    return val && {}.toString.call(val) === '[object Function]'
  },
  getDotNotationVal (obj, prop, val) { // 获取形如a.b.c value
    if (!prop) {
      return obj
    }
    const props = prop.split('.')
    const final = props.pop()
    let key
    while ((key = props.shift())) {
      if (typeof obj[key] === 'undefined') {
        return undefined
      }
      obj = obj[key]
    }
    return val ? (obj[final] = val) : obj[final]
  },
  substring (value, len, hasDot = true) { // 字串截取（含中文）
    let newLength = 0
    let newStr = ''
    const chineseRegex = /[^\x00-\xff]/g
    let singleChar = ''
    const strLength = value.replace(chineseRegex, '**').length
    for (let i = 0; i < strLength; i++) {
      singleChar = value.charAt(i).toString()
      if (singleChar.match(chineseRegex) != null) {
        newLength += 2
      } else {
        newLength++
      }
      if (newLength > len) {
        break
      }
      newStr += singleChar
    }
    if (hasDot && strLength > len) {
      newStr += '...'
    }
    return newStr
  }
}
