// https://github.com/joakimbeng/kebab-case
const KEBAB_REGEX = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g
const REVERSE_REGEX = /-[a-z\u00E0-\u00F6\u00F8-\u00FE]/g
const kebabCase = str => {
  return str.replace(KEBAB_REGEX, match => {
    return `-${match.toLowerCase()}`
  })
}
const reverse = str => {
  return str.replace(REVERSE_REGEX, match => {
	  return match.slice(1).toUpperCase()
  })
}

export const domStyle = (el, props, val, pseudo) => {
  if (!el || !props) {
    return
  }
  if (props === Object(props)) {
    for (let prop in props) {
      domStyle(el, kebabCase(prop), props[prop])
    }
    return props
  }
  const context = (el.ownerDocument || document)['defaultView']
  // if (!context.hasOwnProperty('getComputedStyle')) {
  if (typeof context.getComputedStyle !== 'function') {
    context.getComputedStyle = (el, pseudo) => {
      return {
        el,
        getPropertyValue (prop) {
          prop === 'float' && (prop = 'styleFloat')
          prop = reverse(prop)
          return el.currentStyle ? el.currentStyle[prop] : el.style[prop]
        }
      }
    }
  }
  if (typeof val !== 'undefined') {
    el.style[reverse(props === 'float' ? 'cssFloat' : props)] = val || ''
    return val
  }
  return context.getComputedStyle(el, pseudo || null).getPropertyValue(kebabCase(props === 'cssFloat' ? 'float' : props))
}

export const getOffsetParent = (el, root = window) => {
  // NOTE: 1 DOM access here
  var offsetParent = el.offsetParent
  return offsetParent === root.document.body || !offsetParent ? root.document.documentElement : offsetParent
}

export const getScrollParent = (el, root = window) => {
  const parent = el.parentNode
  if (!parent) {
    return el
  }
  if (parent === root.document) {
    // Firefox puts the scrollTop value on `documentElement` instead of `body`, we then check which of them is
    // greater than 0 and return the proper element
    if (root.document.body.scrollTop || root.document.body.scrollLeft) {
        return root.document.body
    } else {
        return root.document.documentElement
    }
  }
  // Firefox want us to check `-x` and `-y` variations as well
  for (let style of ['overflow', 'overflow-x', 'overflow-y']) {
    const key = domStyle(parent, style)
    if (['scroll', 'auto'].includes(key)) {
      return parent
    }
  }
  // If the detected scrollParent is body, we perform an additional check on its parentNode
  // in this way we'll get body if the browser is Chrome-ish, or documentElement otherwise
  // fixes issue #65
  return getScrollParent(el.parentNode)
}
